import { system, world, ItemStack } from "@minecraft/server";
import { Util } from "../../library/utilities/engineUtil";
import { PlayerData } from "../entity/player";
import { VerticalSlabSet } from "../block/verticalSlabSet";

export class VerticalSlabWaterlog {
    #canWaterlog = true;
    success = false;

    constructor() {
        this.item;
        this.waterlogData;
        this.player;
    }
    
    subscribe() {
        world.beforeEvents.itemUseOn.subscribe(data => {
            this.waterlogData = new VerticalSlabSet(data);
            this.player = new PlayerData(data.source);
            this.item = this.waterlogData.itemId;
            this.#waterlog(data);
        });
        world.afterEvents.playerPlaceBlock.subscribe(() => this.#onPlayerPlaceBlock());
    }

    #waterlog(data) {
        this.#canWaterlog = true;
        const targetToWaterlog = this.waterlogData.evaluateWaterlogCondition();
        if (targetToWaterlog != undefined) {
            const waterlogLoc = targetToWaterlog.location;
            const aimLoc = this.waterlogData.aimBlock.location;
            this.success = false;
            if(JSON.stringify(waterlogLoc) == JSON.stringify(aimLoc) && this.#canWaterlog) {
                system.run(() => {
                    this.success = targetToWaterlog.waterlog();
                });
            } else if(JSON.stringify(waterlogLoc) == JSON.stringify(this.waterlogData.location)) {
                data.cancel = true;
                system.run(() => {
                    this.success = targetToWaterlog.waterlog();
                });
            };
            system.run(() => {
                if(this.success) {
                    world.playSound("bucket.empty_water", Util.transToWorldLoc(targetToWaterlog.location));
                    this.player.setMainhandItem(new ItemStack("minecraft:bucket", 1));
                };
            });
        }
    }

    #onPlayerPlaceBlock() {
        if (this.item == "minecraft:water_bucket") {
            this.#canWaterlog = false;
        };
    }
}

export const onWaterlog = new VerticalSlabWaterlog();